"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useListTransition = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const insertArray_1 = require("../helpers/insertArray");
const setAnimationFrameTimeout_1 = require("../helpers/setAnimationFrameTimeout");
function useListTransition(list, timeout) {
    const keyRef = (0, react_1.useRef)(0);
    // change list to our list form with extra information.
    const initialList = list.map((item, key) => ({
        item,
        key: keyRef.current,
        stage: 'enter',
    }));
    const [listState, setListState] = (0, react_1.useState)(initialList);
    (0, react_1.useEffect)(function handleListChange() {
        let newItemsWithIndex = [];
        list.forEach((item, index) => {
            if (listState.every((itemState) => itemState.item !== item)) {
                newItemsWithIndex.push({ item, index });
            }
        });
        // 1 add new items into list state
        if (newItemsWithIndex.length > 0) {
            keyRef.current++;
            setListState((prevListState) => newItemsWithIndex.reduce((prev, { item, index }, i) => (0, insertArray_1.insertArray)(prev, index, {
                item,
                key: keyRef.current,
                stage: 'from',
            }), prevListState));
        }
        // 2 enter those new items immediatly
        if (newItemsWithIndex.length === 0 &&
            listState.some((item) => item.stage === 'from')) {
            (0, setAnimationFrameTimeout_1.setAnimationFrameTimeout)(() => {
                setListState((prev) => prev.map((item) => (Object.assign(Object.assign({}, item), { stage: item.stage === 'from' ? 'enter' : item.stage }))));
            });
        }
        // 3 leave items from list state
        const subtractItemStates = listState.filter((itemState) => !list.includes(itemState.item) && itemState.stage !== 'leave');
        const subtractItems = subtractItemStates.map((item) => item.item);
        if (newItemsWithIndex.length === 0 && subtractItemStates.length > 0) {
            setListState((prev) => prev.map((itemState) => subtractItemStates.includes(itemState)
                ? Object.assign(Object.assign({}, itemState), { stage: 'leave' }) : itemState));
            (0, setAnimationFrameTimeout_1.setAnimationFrameTimeout)(() => {
                setListState((prev) => prev.filter((item) => !subtractItems.includes(item.item)));
            }, timeout);
        }
    }, [list, listState, timeout]);
    function transition(renderCallback) {
        return listState.map((item) => ((0, jsx_runtime_1.jsx)(react_1.Fragment, { children: renderCallback(item.item, item.stage) }, item.key)));
    }
    return transition;
}
exports.useListTransition = useListTransition;
//# sourceMappingURL=index.js.map